//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : HashGraph
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   The HashGraph class implements the Graph interface, which includes a
//  large number of methods to query/update the nodes and edges in the
//  Graph. It uses a variety of collection classes to do most of the work.
//  It stores two primary maps:
//
//    nodeMap: a map from a node name to its associated LocalInformation
//             (sets of incoming/outgoing nodes and edges)
//    edgemap: a map from an edge to it associated values
//
// Note that each Edged object is immutable and has three references to
// it: one in the edgeMap, and one each in its origin node's outEdges
// set and its destination node's inEdges set.
//
//   This duplication of information allows for faster processing in many
// methods, but has the overhead that methods that add/remove nodes/edges
// must to more work: changing the main and local data structures.
//
// Known Bugs     : None
//
// Future Plans   : None
//
// Program History:
//   11/27/01: R. Pattis - Operational in Java for 15-111
//   11/16/04: R. Pattis - Generalized to Object edge values
//                         Added main edgeMap; simplefied Edge class
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


package edu.cmu.cs.pattis.cs151xx;


import java.util.StringTokenizer;
import java.util.Comparator;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Arrays;
import java.util.Collections;

import java.io.EOFException;



/**
* The HashGraph class implements the Graph interface, which includes a
*   large number of methods to query/update the nodes and edges in the
*   Graph.
* It uses a variety of collection classes to do most of the work.
* It stores two primary maps:
*   <ul>
*     <li>nodeMap: a map from a node name to its associated LocalInformation
*          (sets of incoming/outgoing nodes and edges)
*     <p>
*     <li>edgemap: a map from an edge to it associated values
*   </ul>
* Note that each Edged object is immutable and has three references to
*   it: one in the edgeMap, and one each in its origin node's outEdges
*   set and its destination node's inEdges set.
* <p>
* This duplication of information allows for faster processing in many
*   methods, but has the overhead that methods that add/remove nodes/edges
* must to more work: changing the main and local data structures.
*/
public class HashGraph implements Graph {
  
  
  /**
  * Construct a new Graph, initially storing no nodes and no edges.
  *
  * @param evio - an object with methods for performing input/output on
  *               the type of edge values used in this graph.
  */ 
  public HashGraph(EdgeValueIO evio)
  {
  }
 
 
 
  /**
  * Construct a new Graph, initially storing the same nodes and edges as 
  *   its parameter.
  *
  * @param g - the Graph to copy.
  */ 
  public HashGraph(Graph g)
  {
  }

 
 
  /**
  * Remove all the nodes and edges from this graph.
  */ 
  public void clear ()
  {
  }
  
  
  
  /**
  * Add a node to this graph, returning the new graph.
  * If the node is already in the graph, make no changes to the graph;
  *   otherwise, add the node (associated with empty LocalInformation).
  *
  * @param nodeName - the name of the node to add to this graph.
  *
  * @returns the new graph.
  */ 
  public Graph addNode (String nodeName)
  {
    return this;
  }
  
  
 
  /**
  * Add an edge to this graph, returning the new graph.
  * If the edge is already in the graph, change its value to this value;
  *   otherwise, add a new edge.
  * Note that if the origin or destination nodes are not in the graph,
  *   add them as well.
  * All appropriate LocalInformation for the origin and destination nodes is updated.
  *
  * @param edge    - the edge to add to this graph.
  * @param value - the value of the edge.
  *
  * @returns the new graph.
  */ 
  public Graph addEdge (Graph.Edge edge, Object edgeValue)
  {
    return this;
  }  
  


  /**
  * Add an edge to this graph, returning the new graph.
  * If the edge is already in the graph, change its value to this value;
  *   otherwise, add a new edge.
  * Note that if the origin or destination nodes are not in the graph,
  *   add them as well.
  * All appropriate LocalInformation for the origin and destination nodes is updated.
  *
  * @param originNodeName      - the origin node for the edge.
  * @param destinationNodeName - the destination node for the edge.
  * @param edgeValue           - the value of the edge.
  *
  * @returns the new graph.
  */ 
  public Graph addEdge (String originNodeName, String destinationNodeName, Object edgeValue)
  {return this;}
  
  
  
  /**
  * Remove a node from this graph, returning the new graph.
  * If the node is in the graph, remove all edges connected
  *   to it; if not, make no changes.
  *
  * @param nodeName - the name of the node to remove from this graph.
  *
  * @returns the new graph.
  */ 
  public Graph removeNode (String nodeName)
  {
    return this;
  }
  
  
  
  /**
  * Remove an edge from this graph, returning the new graph.
  * Update the LocalInformation for the origin/destination nodes of
  *   this edge.
  *
  * @param edge  - the edge to remove from this graph.
  *
  * @returns the new graph.
  */ 
  public Graph removeEdge (Graph.Edge edge)
  {
    return this;
  }  


    
  /**
  * Remove an edge from this graph, returning the new graph.
  * Update the LocalInformation for the origin/destination nodes of
  *   this edge.
  *
  * @param originNodeName      - the origin node for the edge to remove from the graph.
  * @param destinationNodeName - the destination node for the edge to remove from the graph.
  *
  * @returns the new graph.
  */ 
  public Graph removeEdge (String originNodeName, String destinationNodeName)
  {return this;}

    
  /**
  * Read this graph from a file, one line at at time, in the form:
  *   <b>origin destination value</b> (where each is separated from the other by
  *   tokenSeparator).
  * If this graph already contains nodes/edges, these are added; if you
  *   want to discard the old graph, call clear first.
  * Note that the EdgeValueIO object supplied to the constructor of this
  *   object is used on the value.
  *
  * @param input         - file open for reading.
  * @param tokeSeparator - character to use to separate origin, destination, and value.
  *
  * @returns the new graph.
  */ 
  public Graph load (TypedBufferReader input, char tokenSeparator)
  {
    return this;
  }  


    
  /**
  * Write this graph into a file, one line at a time so that it can be read by the load
  *   method (in the form origin, destination, value; where each is
  *  separated from the other by tokenSeparator).
  *
  * @param output        - file open for writing.
  * @param tokeSeparator - character to use to separate origin, destination, and value.
  */ 
  public void write (TypedBufferWriter output, char tokenSeparator)
  {
  }  



  /**
  * Returns the number of nodes in the Graph.
  *
  * @returns the number of nodes in the Graph.
  */
	public int getNodeCount ()
	{return 0;}


	
  /**
  * Returns the number of edges in the Graph.
  *
  * @returns the number of edges in the Graph.
  */
  public int getEdgeCount ()
	{return 0;}
	
	
	
  /**
  * Return the EdgeValueIO object for this graph.
  *
  * @returns the EdgeValueIO object for this graph.
  */ 
  public EdgeValueIO getEdgeValueIO ()
  {return null;}
  
  
  
  /**
  * Return whether or not this graph stores a node.
  *
  * @param nodeName - the name of the node to check.
  *
  * @returns whether or not this graph stores a node.
  */ 
  public boolean hasNode (String nodeName)
  {return false;}
  
  
  
  /**
  * Returns whether or not this graph stores an edge with the specified
  *   edge.
  *
  * @param edge - the edge to check.
  *
  * @returns whether or not this graph stores an edge with the specified
  *   origin and destination.
  */ 
  public boolean hasEdge (Graph.Edge edge)
  {return false;}
  
  
  
  /**
  * Returns whether or not this graph stores an edge with the specified
  *   origin and destination.
  *
  * @param originNodeName      - the origin node for the edge to check.
  * @param destinationNodeName - the destination node for the edge to check.
  *
  * @returns whether or not this graph stores an edge with the specified
  *   origin and destination.
  */ 
  public boolean hasEdge (String originNodeName, String destinationNodeName)
  {return false;}
  
  
  
  /**
  * Returns the value associated with this edge (null if the edge is not
  *   in this graph).
  *
  * @param edge- the edge whose associated value is to be retrieved.
  *
  * @returns the value associated with this edge (null if the edge is not
  *   in this graph.
  */ 
  public Object getEdgeValue (Graph.Edge edge)
  {return null;}
  
  
  
  /**
  * Returns the value associated with the edge with the specified
  *   origin and destination.
  *
  * @param origin      - the origin node of the edge whose associated value is to be retrieved.
  * @param destination - the destination of the edge whose associated value is to be retrieved.
  *
  * @returns the value associated with the edge with the specified
  *   origin and destination.
  */ 
  public Object getEdgeValue (String origin, String destination)
  {return null;}
  

  
  /**
  * Returns the in-degree of the specified node.
  * If the node is not present in the graph, it returns 0.
  *
  * @param nodeName - the name of the node to check.
  *
  * @returns the in-degree of the specified node.
  */ 
  public int inDegree (String nodeName)
  {
    return 0;
  }
  
  
  
  /**
  * Returns the out-degree of the specified node.
  * If the node is not present in the graph, it returns 0.
  *
  * @param nodeName - the name of the node to check.
  *
  * @returns the in-degree of the specified node.
  */ 
  public int outDegree (String nodeName)
  {
    return 0;
  }
  


  /**
  * Returns the degree of the specified node.
  * If the node is not present in the graph, it returns 0.
  * Otherwise, it returns the sum of the in-degree and out-degree
  *
  * @param nodeName - the name of the node to check.
  *
  * @returns the in-degree of the specified node.
  */ 
  public int degree (String nodeName)
  {return 0;}
  


  /**
  * Returns an UNMODIFIABLE set containing the names of all the nodes
  *   in the graph.
  *
  * @returns an UNMODIFIABLE set containing the names of all the nodes
  *   in the graph.
  */ 
  public Set getAllNodes()
  {return null;}
  


  /**
  * Returns an UNMODIFIABLE set containing all the edges in the graph.
  *
  * @returns an UNMODIFIABLE set containing all the edges in the graph.
  */ 
  public Set getAllEdges()
  {return null;}
  
  
  
  /**
  * Returns an UNMODIFIABLE set containing the destination nodes reachable
  *  from <code>originNodeName</code> in the graph.
  *
  * @param originNodeName - the name of the origin node
  *
  * @returns an UNMODIFIABLE set containing the destination nodes reachable
  *  from <code>originaNodeName</code> in the graph.
  */ 
  public Set getOutNodes (String originNodeName)
  {
    return null;
  }
  


  /**
  * Returns an UNMODIFIABLE set containing the origin nodes that reach
  *  to <code>destinationNodeName</code> in the graph.
  *
  * @param destinationNodeName - the name of the origin node
  *
  * @returns an UNMODIFIABLE set containing the origin nodes that reach
  *  to <code>destinationNodeName</code> in the graph.
  */ 
  public Set getInNodes (String destinationNodeName)
  {
    return null;
  }
  


  /**
  * Returns an UNMODIFIABLE set containing all the edges in the graph whose
  *  origin node is <code>originNodeName</code>.
  *
  * @param originNodeName - the name of the origin node for the edges
  *
  * @returns an UNMODIFIABLE set containing all the edges in the graph whose
  *  origin node is <code>originNodeName</code>.
  */ 
  public Set getOutEdges (String originNodeName)
  {
    return null;
  }
  
  
  
  /**
  * Returns an UNMODIFIABLE set containing all the edges in the graph whose
  *  destination node is <code>destinationNodeName</code>.
  *
  * @param destinationNodeName - the name of the destination node for the edges
  *
  * @returns an UNMODIFIABLE set containing all the edges in the graph whose
  *  destination node is <code>destinationNodeName</code>.
  */ 
  public Set getInEdges (String destinationNodeName)
  {
    return null;
  }
  
 
  
  /**
  * Returns a String representation for this graph.
  * It is useful for seeing the graph, not writing it into a file.
  *
  * @returns a String representation for this graph.
  */
	public String toString()
	{
	  Object[] nodeNames = getAllNodes().toArray();
	  Arrays.sort(nodeNames);  //Use comparable
	  StringBuffer answer = new StringBuffer();
	  for (int i=0; i<nodeNames.length; i++)
	    answer.append("\n\nNode = " + nodeNames[i] + "\n" + nodeMap.get(nodeNames[i]));
	    
	  return answer.toString();	  
	}
	
	
	/**
	* Map: String -> LocalInformation
	*/
  private Map nodeMap;
  
	/**
	* Map: Edge -> Object
	*/
  private Map edgeMap;              //Map: edge -> Value
  
	/**
	* For parsing/unparsing edge values when doing I/O
	*/
  private EdgeValueIO evio;         //For parsing/unparsing edge values
  
  
  /**
  * Edge is an immutable inner class for storing edges in a graph; each consists
  *   of the origin and desination node in that graph.
  */
  private class Edge implements Graph.Edge {

    /**
    * Construct a new, immutable Edge from its parameters.
    *
    * @param origin      - the name of the origin node for the edge
    * @param destination - the name of the destination node for the edge
    *
    */ 
    private Edge(String origin, String destination)
    {
      this.origin      = origin;
      this.destination = destination;
    }
    
    
    /**
    * Returns whether this edge is equal to another.
    * Two edges are equal if they have the same origin
    *   and destination nodes.
    *
    * @param o - the Edge to compare this one to.
    *
    * @returns whether this edge is equal to another.
    */ 
    public boolean equals(Object o)
    {
      if (this == o)
        return true;
      if (!(o instanceof Graph.Edge))
        return false;
      
      Graph.Edge ge = (Graph.Edge)o;
      return origin.equals(ge.getOrigin()) && destination.equals(ge.getDestination());
    }
    
    /**
    * Returns the hash code for this Edge.
    *
    * @returns the hash code for this Edge.
    */ 
    public int hashCode()
    {return origin.hashCode() * destination.hashCode();}
    
    
    /**
    * Returns the origin node for this Edge.
    *
    * @returns the origin node for this Edge.
    */ 
    public String getOrigin()
    {return origin;}
    
    /**
    * Returns the destination node for this Edge.
    *
    * @returns the destination node for this Edge.
    */ 
    public String getDestination()
    {return destination;}
    
    /**
    * Returns the value associated with this Edge.
    *
    * @returns the value associated with this Edge.
    */ 
    public Object getValue()
    {return edgeMap.get(this);}
    
    
    /**
    * Return a String repesentation of the Edge in the form:
    *   origin->destination if the edge's value is null, and
    *   origin->destination(value) if the edge's value is non-null.
    *
    * @returns a String repesentation of the Edge.
    */ 
    public String toString()
    {
      //Note: Because Edge is a inner (not just nested) class, it can
      //      access the edgeMap of the graph it was constructed in, and
      //      use it to look-up its edge value in this toString method.
      Object edgeValue = getValue();
      return origin+"->"+destination+(edgeValue==null ? "" :"("+edgeValue+")");
    }
    
    
    //Instance variables: all private and immutable
    private final String origin;
    private final String destination;
  }
 
 
  
  /**
  * LocalInformation is an inner class for storing the nodes/edges that are associated
  *   with a given node: the nodes directly leading to/from it, and the edges having
  *   it as an origin/destination.
  */
  public class LocalInformation {

    /**
    * Construct a new LocalInformation, with no nodes/edges.
    */ 
    public LocalInformation()
    {
      outNodes = new HashSet();
      inNodes  = new HashSet();
      outEdges = new HashSet();
      inEdges  = new HashSet();
    }

    /**
    * Return a String repesentation of the LocalInformation.
    * Everything is sorted by node name.
    *
    * @returns a String repesentation of the LocalInformation.
    */ 
    public String toString()
    {
      Object[] o = outEdges.toArray();
      Arrays.sort(o,destOrder);
      StringBuffer answer = new StringBuffer("  Out Edges: ");
      for (int i=0; i<outEdges.size(); i++)
        answer.append(o[i] + (i<outEdges.size()-1 ? ", " : ""));
      
      /*reuse same Object[]*/ o = inEdges.toArray();
      Arrays.sort(o,orgOrder);
      answer.append("\n  In  Edges: ");
      for (int i=0; i<inEdges.size(); i++)
        answer.append(o[i] + (i<inEdges.size()-1 ? ", " : ""));
      
      return answer.toString();
    }
    
    
    //Available to outer class only
    public final Set outNodes;
    public final Set inNodes;
    public final Set outEdges;
    public final Set inEdges;
  }
  
  //Used in toString in LocalInformation only
  private static Comparator destOrder = new Comparator() {
	  public int compare(Object o1, Object o2)
	  {return ((Edge)o1).destination.compareTo(((Edge)o2).destination);}
  };
  
  private static Comparator orgOrder  = new Comparator() {
  	public int compare(Object o1, Object o2)
	  {return ((Edge)o1).origin.compareTo(((Edge)o2).origin);}
  };
  
  
  
  
  
  
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Driver for this class, written as main in this class
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
  
  
  
  private static char menuPrompt ()
  {
    System.out.println("\n\nAccessors              Mutator           General");
    System.out.println("  h - hasNode            n - addNode       l - load from file");
    System.out.println("  H - hasEdge            e - addEdge       w - write to file");
    System.out.println("  v - edgeValue          N - removeNode    C - copy constructor");
    System.out.println("  > - inDegree           E - removeEdge    q - quit");  
    System.out.println("  < - outDegree          c - clear");
    System.out.println("  d - degree");
    System.out.println("  g - getAllNodes");
    System.out.println("  G - getAllEdges");
    System.out.println("  # - node/edge counts");
    System.out.println("  o - out information");
    System.out.println("  i - in information");
    System.out.println("  t - toString");
    
    return Prompt.forChar("\nEnter Command","hHv><dgG#oitneNEclwCq");
  }





	public static void main(String[] args)
	{
    Graph g = new HashGraph(new IntegerEdgeValue());
 
	  //Test graph object using any method
	  for (;;) try {
	    char selection = menuPrompt();
	    
      if (selection == 'h') {
        String nodeName  = Prompt.forString("  Enter node");
        System.out.println("  hasNode("+nodeName+") = " + g.hasNode(nodeName));
      }
       
      else if (selection == 'H') {
        String origin      = Prompt.forString("  Enter origin      node");
        String destination = Prompt.forString("  Enter destination node");
        System.out.println("  hasEdge("+origin+"->"+destination+") = " + g.hasEdge(origin,destination));
      }
       
      else if (selection == 'v') {
        String origin      = Prompt.forString("  Enter origin      node");
        String destination = Prompt.forString("  Enter destination node");
        System.out.println("  edgeValue("+origin+"->"+destination+") = " + g.getEdgeValue(origin,destination));
      }
       
      else if (selection == '>') {
        String nodeName  = Prompt.forString("  Enter node");
        System.out.println("  inDegree("+nodeName+") = " + g.inDegree(nodeName));
      }
      
      else if (selection == '<') {
        String nodeName  = Prompt.forString("  Enter node");
        System.out.println("  outDegree("+nodeName+") = " + g.outDegree(nodeName));
      }
      
      else if (selection == 'd') {
        String nodeName  = Prompt.forString("  Enter node");
        System.out.println("  degree("+nodeName+") = " + g.degree(nodeName));
      }
      
      else if (selection == 'g') {
        Set nodes  = g.getAllNodes();
        System.out.print("  getAllNodes() = ");
        for (Iterator it=nodes.iterator(); it.hasNext(); )
          System.out.print(it.next()+" ");
        System.out.println();
			}
			      
      else if (selection == 'G') {
        Set edges  = g.getAllEdges();
        System.out.print("  getAllEdges() = ");
        for (Iterator it=edges.iterator(); it.hasNext(); )
          System.out.print(it.next()+" ");
        System.out.println();
			}
			      
      else if (selection == '#') {
        System.out.println("  getNodeCount() = " + g.getNodeCount());
        System.out.println("  getEdgeCount() = " + g.getEdgeCount());
      }
      
      else if (selection == 'o') {
        String nodeName  = Prompt.forString("  Enter node");
        System.out.print("  getOutNodes("+nodeName+") = ");
        Set answer = g.getOutNodes(nodeName);
        for (Iterator it=answer.iterator(); it.hasNext(); )
          System.out.print(it.next()+" ");
        System.out.println();
        System.out.print("  getOutEdges("+nodeName+") = ");
        answer = g.getOutEdges(nodeName);
        for (Iterator it=answer.iterator(); it.hasNext(); )
          System.out.print(it.next()+" ");
      }
      
      else if (selection == 'i') {
        String nodeName  = Prompt.forString("  Enter node");
        System.out.print("  getInNodes("+nodeName+") = ");
        Set answer = g.getInNodes(nodeName);
        for (Iterator it=answer.iterator(); it.hasNext(); )
          System.out.print(it.next()+" ");
        System.out.println();
        System.out.print("  getInEdges("+nodeName+") = ");
        answer = g.getInEdges(nodeName);
        for (Iterator it=answer.iterator(); it.hasNext(); )
          System.out.print(it.next()+" ");
      }
      
      else if (selection == 'n') {
        String nodeName = Prompt.forString("  Enter node");
        g.addNode(nodeName);
      }
      
      else if (selection == 'e') {
        String origin      = Prompt.forString("  Enter origin      node");
        String destination = Prompt.forString("  Enter destination node");
        String value       = Prompt.forString("  Enter origin->destination value");
        g.addEdge(origin,destination,g.getEdgeValueIO().readEdgeValue(value));
      }
       
      else if (selection == 'N') {
        String nodeName = Prompt.forString("  Enter node");
        g.removeNode(nodeName);
      }
      
      else if (selection == 'E') {
        String origin      = Prompt.forString("  Enter origin      node");
        String destination = Prompt.forString("  Enter destination node");
        g.removeEdge(origin,destination);
      }
       
      else if (selection == 'c')
        g.clear();
       
      else if (selection == 'l')
        g.load(new TypedBufferReader ("  Enter file (containing Graph) name"),' ');

      else if (selection == 'w')
        g.write(new TypedBufferWriter (Prompt.forString("  Enter file (to contain Graph) name")),' ');
      
      else if (selection == 't')
        System.out.println(g);
 
      else if (selection == 'C') {
        Graph g2 = new HashGraph(g);
        g.clear();
        System.out.println("Clearing original Graph");
        System.out.println("Printing copy of Graph");
        System.out.println(g2);
 
      }else if (selection == 'q')
        break;
      
      else
        System.out.println("\""+selection+"\" is unknown command");

		}catch (Exception e) {
	    e.printStackTrace();
	  }
  }

  
  
  
};

