//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : GraphTest
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   This tests any concrete subclass of the Graph interface using the JUnit
// framework. It automatically performs various tests, logging their failures.
// Note that it tests various iterators as well.
//
//   Because the toString method is canonical (equal graphs return equal
// string) we can use it in assertEquals. 
//
// Bugs           : Cannot get load to work (complains about TypedBufferReader)
//
// Future Plans   : Add more tests (of course!)
//
// Program History:
//   11/20/04: R. Pattis - Operational for 15-200
//   11/15/06: R. Pattis - Updated for JUnit 4.1
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


import org.junit.*;                         //For tags
import static org.junit.Assert.*;           //For assertions

import java.util.*;

import edu.cmu.cs.pattis.cs151xx.orderedCollections.*;

import edu.cmu.cs.pattis.cs151xx.Graph;
import edu.cmu.cs.pattis.cs151xx.HashGraph;
import edu.cmu.cs.pattis.cs151xx.EdgeValueIO;
import edu.cmu.cs.pattis.cs151xx.IntegerEdgeValue;



public class GraphTest {

	private Graph g;
	private String keys,values;


  @Before
	public void setUp()
	{
	  g = new HashGraph(new IntegerEdgeValue());
	}
	
	
	
	//////////////////////////////////////////////////
	//////////////////////////////////////////////////
	//
	// Helper Methods
	//
	//////////////////////////////////////////////////
	//////////////////////////////////////////////////
	
	private Set makeSet(Object[] o)
	{
	  Set s = new HashSet();
	  for (int i=0; i<o.length; i++)
	    s.add(o[i]);
	  return s;
	}
	
	//
	// Draw this graph! It is used as the basis for many tests
  //
  	
	private void loadStandardGraph()
	{
    g.addEdge   ("a","b",null);
    g.addEdge   ("a","c","a->c");
    g.addEdge   ("b","d",new Integer(1));
    g.addEdge   ("c","d",new Double(.5));
    g.addEdge   ("a","d",new Integer(0));
    g.addEdge   ("d","a",new Integer(-1));
    g.addNode   ("e");
	}
	

	private void checkEmptyManyWays()
	{
		assertTrue(g.getNodeCount() == 0);
		assertTrue(g.getEdgeCount() == 0);
		assertTrue(g.getAllNodes().size() == 0);
		assertTrue(g.getAllEdges().size() == 0);
	}
	
	
	//////////////////////////////////////////////////
	//////////////////////////////////////////////////
	//
	// Testing Methods
	//
	//////////////////////////////////////////////////
	//////////////////////////////////////////////////
	
	
	@Test
	public void testConstructor2()
	{
	  loadStandardGraph();
	  Graph g2 = new HashGraph(g);
		assertEquals(g.toString(),g2.toString());
	}



	@Test
	public void testClear()
	{
	  loadStandardGraph();
	  
		assertTrue(g.getNodeCount() == 5);
		assertTrue(g.getEdgeCount() == 6);
		
	  g.clear();

	  checkEmptyManyWays();
	}



	@Test
	public void testAddNode()
	{
	  g.clear();

    g.addNode("a");
    g.addNode("b");
    g.addNode("b");
    
 	  assertTrue (g.hasNode("a"));
 	  assertTrue (g.hasNode("b"));
 	  assertFalse(g.hasNode("c"));
	}



	@Test
  public void testAddEdge()
  {
	  g.clear();

    g.addEdge   ("a","b",null);
 	  assertTrue  (g.hasNode("a"));
 	  assertTrue  (g.hasNode("b"));
 	  assertTrue  (g.hasEdge("a","b"));
 	  assertFalse (g.hasEdge("b","a"));
 	  assertFalse (g.hasEdge("b","c"));
 	  
 	  g.addEdge   ("b","c",null);
 	  assertTrue  (g.hasNode("c"));
 	  assertTrue  (g.hasEdge("a","b"));
 	  assertTrue  (g.hasEdge("b","c"));
 	  assertFalse (g.hasEdge("b","a"));
 	  assertFalse (g.hasEdge("a","c"));
 	  
 	  g.addEdge   ("a","c",null);
 	  
 	  assertEquals(2,g.getOutNodes("a").size());
 	  assertEquals(2,g.getOutEdges("a").size());
 	  assertEquals(0,g.getInNodes ("a").size());
 	  assertEquals(0,g.getInEdges ("a").size());
 	  
 	  assertEquals(1,g.getOutNodes("b").size());
 	  assertEquals(1,g.getOutEdges("b").size());
 	  assertEquals(1,g.getInNodes ("b").size());
 	  assertEquals(1,g.getInEdges ("b").size());

 	  assertEquals(0,g.getOutNodes("c").size());
 	  assertEquals(0,g.getOutEdges("c").size());
 	  assertEquals(2,g.getInNodes ("c").size());
 	  assertEquals(2,g.getInEdges ("c").size());
  }



 	@Test
  public void testRemoveNode()
  {
	  loadStandardGraph();

    g.removeNode("b");
    
 	  assertTrue  (g.hasNode("a"));
 	  assertTrue  (g.hasNode("c"));
 	  assertTrue  (g.hasNode("d"));
 	  assertTrue  (g.hasNode("e"));
 	  
 	  assertTrue  (g.hasEdge("a","c"));
 	  assertTrue  (g.hasEdge("a","d"));
 	  assertTrue  (g.hasEdge("c","d"));
 	  assertTrue  (g.hasEdge("d","a"));
 	  
 	  assertFalse (g.hasEdge("a","b"));
 	  assertFalse (g.hasEdge("b","d"));
 	  
 	  assertEquals(2,g.getOutNodes("a").size());
 	  assertEquals(2,g.getOutEdges("a").size());
 	  assertEquals(1,g.getInNodes ("a").size());
 	  assertEquals(1,g.getInEdges ("a").size());
 	  
 	  assertEquals(0,g.getOutNodes("b").size());
 	  assertEquals(0,g.getOutEdges("b").size());
 	  assertEquals(0,g.getInNodes ("b").size());
 	  assertEquals(0,g.getInEdges ("b").size());
 	  
 	  assertEquals(1,g.getOutNodes("c").size());
 	  assertEquals(1,g.getOutEdges("c").size());
 	  assertEquals(1,g.getInNodes ("c").size());
 	  assertEquals(1,g.getInEdges ("c").size());

 	  assertEquals(1,g.getOutNodes("d").size());
 	  assertEquals(1,g.getOutEdges("d").size());
 	  assertEquals(2,g.getInNodes ("d").size());
 	  assertEquals(2,g.getInEdges ("d").size());
  }



	@Test
  public void testRemoveEdge()
  {
	  loadStandardGraph();

    g.removeEdge ("a","b");
    g.removeEdge ("b","d");
    g.removeEdge ("d","b");  //not present
    
 	  assertTrue  (g.hasNode("a"));
 	  assertTrue  (g.hasNode("b"));
 	  assertTrue  (g.hasNode("c"));
 	  assertTrue  (g.hasNode("d"));
 	  assertTrue  (g.hasNode("e"));
 	  
 	  assertTrue  (g.hasEdge("a","c"));
 	  assertTrue  (g.hasEdge("a","d"));
 	  assertTrue  (g.hasEdge("c","d"));
 	  assertTrue  (g.hasEdge("d","a"));
 	  
 	  assertFalse (g.hasEdge("a","b"));
 	  assertFalse (g.hasEdge("b","d"));
 	  
 	  assertEquals(2,g.getOutNodes("a").size());
 	  assertEquals(2,g.getOutEdges("a").size());
 	  assertEquals(1,g.getInNodes ("a").size());
 	  assertEquals(1,g.getInEdges ("a").size());
 	  
 	  assertEquals(0,g.getOutNodes("b").size());
 	  assertEquals(0,g.getOutEdges("b").size());
 	  assertEquals(0,g.getInNodes ("b").size());
 	  assertEquals(0,g.getInEdges ("b").size());
 	  
 	  assertEquals(1,g.getOutNodes("c").size());
 	  assertEquals(1,g.getOutEdges("c").size());
 	  assertEquals(1,g.getInNodes ("c").size());
 	  assertEquals(1,g.getInEdges ("c").size());

 	  assertEquals(1,g.getOutNodes("d").size());
 	  assertEquals(1,g.getOutEdges("d").size());
 	  assertEquals(2,g.getInNodes ("d").size());
 	  assertEquals(2,g.getInEdges ("d").size());
  }



	@Test
  public void testGetNodeCount()
  {
	  loadStandardGraph();
  
    assertEquals(5, g.getNodeCount());
    
    g.removeNode("f");  //not present
    assertEquals(5, g.getNodeCount());

    g.removeNode("a");
    assertEquals(4, g.getNodeCount());

    g.removeNode("b");
    assertEquals(3, g.getNodeCount());

    g.removeNode("c");
    assertEquals(2, g.getNodeCount());

    g.removeNode("d");
    assertEquals(1, g.getNodeCount());

    g.removeNode("e");
    assertEquals(0, g.getNodeCount());
  }



	@Test
  public void testGetEdgeCount()
  {
	  loadStandardGraph();
  
    assertEquals(6, g.getEdgeCount());
    
    g.removeEdge("d","c");  //not present
    assertEquals(6, g.getEdgeCount());

    g.removeEdge("a","b");
    assertEquals(5, g.getEdgeCount());

    g.removeEdge("a","c");
    assertEquals(4, g.getEdgeCount());

    g.removeEdge("a","d");
    assertEquals(3, g.getEdgeCount());

    g.removeEdge("b","d");
    assertEquals(2, g.getEdgeCount());

    g.removeEdge("c","d");
    assertEquals(1, g.getEdgeCount());

    g.removeEdge("d","a");
    assertEquals(0, g.getEdgeCount());
  }



	@Test
  public void testHasNode()
  {
	  loadStandardGraph();
  
    assertTrue(g.hasNode("a"));
    assertTrue(g.hasNode("b"));
    assertTrue(g.hasNode("c"));
    assertTrue(g.hasNode("d"));
    assertTrue(g.hasNode("e"));
    
    assertFalse(g.hasNode("f"));  //Not present
  }



	@Test
  public void testHasNode2()
  {
	  loadStandardGraph();
  
    for (Iterator i = g.getAllNodes().iterator(); i.hasNext(); )
      assertTrue(g.hasNode((String)i.next()));
    for (Iterator i = g.getAllEdges().iterator(); i.hasNext(); ) {
      Graph.Edge e = (Graph.Edge)i.next();
      assertTrue(g.hasNode(e.getOrigin()));
      assertTrue(g.hasNode(e.getDestination()));
    }
  }



	@Test
  public void testHasEdge()
  {
	  loadStandardGraph();
  
    assertTrue(g.hasEdge("a","b"));
    assertTrue(g.hasEdge("a","c"));
    assertTrue(g.hasEdge("a","d"));
    assertTrue(g.hasEdge("b","d"));
    assertTrue(g.hasEdge("c","d"));
    assertTrue(g.hasEdge("d","a"));
    
    assertFalse(g.hasEdge("b","a"));
    assertFalse(g.hasEdge("c","a"));
    assertFalse(g.hasEdge("d","b"));
    assertFalse(g.hasEdge("d","c"));
    assertFalse(g.hasEdge("a","e"));
  }



	@Test
  public void testHasEdge2()
  {
	  loadStandardGraph();
  
    Set allEdges = g.getAllEdges();
    for (Iterator i = g.getAllEdges().iterator(); i.hasNext(); ) {
      Graph.Edge e = (Graph.Edge)i.next();
      assertTrue(g.hasNode(e.getOrigin()));
      assertTrue(g.hasNode(e.getDestination()));
      assertTrue(g.hasEdge((Graph.Edge)i.next()));
    }
  }



	@Test
  public void testGetEdgeValue()
  {
	  loadStandardGraph();
  
    assertEquals(null,g.getEdgeValue("a","b"));
    assertEquals("a->c",g.getEdgeValue("a","c"));
    assertEquals(new Integer(0),g.getEdgeValue("a","d"));
    assertEquals(new Integer(1),g.getEdgeValue("b","d"));
    assertEquals(new Double(.5),g.getEdgeValue("c","d"));
    assertEquals(new Integer(-1),g.getEdgeValue("d","a"));
    
    assertEquals(null,g.getEdgeValue("d","b"));
    assertEquals(null,g.getEdgeValue("a","e"));
  }



	@Test
  public void testGetEdgeValue2()
  {
	  loadStandardGraph();
  
    Set allEdges = g.getAllEdges();
    for (Iterator i = g.getAllEdges().iterator(); i.hasNext(); ) {
      Graph.Edge e = (Graph.Edge)i.next();
      if (e.getValue() == null)
        assertTrue(g.getEdgeValue(e) == null);
      else
        assertTrue(e.getValue().equals(g.getEdgeValue(e)));
    }
  }



	@Test
  public void testInDegree()
  {
	  loadStandardGraph();
  
    assertEquals(1,g.inDegree("a"));
    assertEquals(1,g.inDegree("b"));
    assertEquals(1,g.inDegree("c"));
    assertEquals(3,g.inDegree("d"));
    assertEquals(0,g.inDegree("e"));  //Separate
    
    assertEquals(0,g.inDegree("f"));  //Not present
  }



	@Test
  public void testOutDegree()
  {
	  loadStandardGraph();
  
    assertEquals(3,g.outDegree("a"));
    assertEquals(1,g.outDegree("b"));
    assertEquals(1,g.outDegree("c"));
    assertEquals(1,g.outDegree("d"));
    assertEquals(0,g.outDegree("e"));  //Separate
    
    assertEquals(0,g.outDegree("f"));  //Not present
  }



	@Test
  public void testDegree()
  {
	  loadStandardGraph();
  
    assertEquals(4,g.degree("a"));
    assertEquals(2,g.degree("b"));
    assertEquals(2,g.degree("c"));
    assertEquals(4,g.degree("d"));
    assertEquals(0,g.degree("e"));  //Separate
    
    assertEquals(0,g.degree("f"));  //Not present
  }



	@Test
  public void testGetAllNodes()
  {
	  loadStandardGraph();
  
    Set nodes = g.getAllNodes();
    
    assertEquals(5,nodes.size());
    
    Set names = makeSet(new Object[]{"a", "b", "c", "d", "e"});
    
    assertEquals(nodes,names);
    
    try {
      nodes.remove("a");
      fail("modification of returned set failed to throw UnsupportedOperationException");
    } catch (UnsupportedOperationException uoe) {}
  }



	@Test
  public void testGetAllEdges()
  {
	  loadStandardGraph();
  
    Set edges = g.getAllEdges();
    
    assertEquals(6,edges.size());
    
    assertTrue(g.hasEdge("a","b"));    
    assertTrue(g.hasEdge("a","c"));    
    assertTrue(g.hasEdge("a","d"));    
    assertTrue(g.hasEdge("b","d"));    
    assertTrue(g.hasEdge("c","d"));    
    assertTrue(g.hasEdge("d","a"));    
    
    try {
      edges.remove("a");
      fail("modification of returned set failed to throw UnsupportedOperationException");
    } catch (UnsupportedOperationException uoe) {}
  }



	@Test
  public void testGetOutNodes()
  {
	  loadStandardGraph();
  
    assertEquals(makeSet(new Object[]{"b","c","d"}), g.getOutNodes("a"));
    assertEquals(makeSet(new Object[]{"d"        }), g.getOutNodes("b"));
    assertEquals(makeSet(new Object[]{"d"        }), g.getOutNodes("c"));
    assertEquals(makeSet(new Object[]{"a"        }), g.getOutNodes("d"));
    assertEquals(makeSet(new Object[]{           }), g.getOutNodes("e"));
    
    try {
      g.getOutNodes("a").remove("a");
      fail("modification of returned set failed to throw UnsupportedOperationException");
    } catch (UnsupportedOperationException uoe) {}
  }



	@Test
  public void testGetInNodes()
  {
	  loadStandardGraph();
  
    assertEquals(makeSet(new Object[]{"d"        }), g.getInNodes("a"));
    assertEquals(makeSet(new Object[]{"a"        }), g.getInNodes("b"));
    assertEquals(makeSet(new Object[]{"a"        }), g.getInNodes("c"));
    assertEquals(makeSet(new Object[]{"a","b","c"}), g.getInNodes("d"));
    assertEquals(makeSet(new Object[]{           }), g.getInNodes("e"));
    
    try {
      g.getInNodes("a").remove("a");
      fail("modification of returned set failed to throw UnsupportedOperationException");
    } catch (UnsupportedOperationException uoe) {}
  }



	@Test
  public void testGetOutEdges()
  {
	  loadStandardGraph();
  
    for (Iterator n=g.getAllNodes().iterator(); n.hasNext(); ) {
      String node = (String)n.next();
      Set eSet = g.getOutEdges(node);
      Set nSet = new HashSet();
      for (Iterator e = eSet.iterator(); e.hasNext(); )
        nSet.add( ((Graph.Edge)e.next()).getDestination() );
      assertEquals(g.getOutNodes(node),nSet);
    }
    
    try {
      g.getOutEdges("a").remove("a");
      fail("modification of returned set failed to throw UnsupportedOperationException");
    } catch (UnsupportedOperationException uoe) {}
  }



	@Test
  public void testGetInEdges()
  {
	  loadStandardGraph();
  
    for (Iterator n=g.getAllNodes().iterator(); n.hasNext(); ) {
      String node = (String)n.next();
      Set eSet = g.getInEdges(node);
      Set nSet = new HashSet();
      for (Iterator e = eSet.iterator(); e.hasNext(); )
        nSet.add( ((Graph.Edge)e.next()).getOrigin() );
      assertEquals(g.getInNodes(node),nSet);
    }
    
    try {
      g.getInEdges("a").remove("a");
      fail("modification of returned set failed to throw UnsupportedOperationException");
    } catch (UnsupportedOperationException uoe) {}
  }



}
  
  
