//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Interface      : Graph
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   This interface specifies the methods for a Graph object, including a
// large number of methods to query/update the nodes and edges in the
// Graph. This interface uses the Set interface to specify a return type
// for many accessors.  
//
// Known Bugs     : None
//
// Future Plans   : None
//
// Program History:
//   11/27/01: R. Pattis - Operational in Java for 15-111
//   11/12/04: R. Pattis - Edges now store general Objects (maybe wrappers)
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


package edu.cmu.cs.pattis.cs151xx;


import java.util.Set;


public interface Graph {

  //Mutators
  
  public void    clear        ();
  
  public Graph   addNode      (String nodeName);
  public Graph   addEdge      (String origin, String destination, Object edgeValue);
  public Graph   addEdge      (Edge edge, Object edgeValue);
  
  public Graph   removeNode   (String nodeName);
  public Graph   removeEdge   (String origin, String destination);
  public Graph   removeEdge   (Edge edge);
  
  public Graph   load         (TypedBufferReader input , char tokenSeparator);
  public void    write        (TypedBufferWriter output, char tokenSeparator);


  //Accessors
  
	public EdgeValueIO getEdgeValueIO ();
	
  public int     getNodeCount ();
  public int     getEdgeCount ();

  public boolean hasNode      (String nodeName);
  public boolean hasEdge      (String origin, String destination);
  public boolean hasEdge      (Edge edge);

  public Object  getEdgeValue (String origin, String destination);
  public Object  getEdgeValue (Edge edge);
  
  public int     inDegree     (String nodeName);
  public int     outDegree    (String nodeName);
  public int     degree       (String nodeName);
  
  //The returned sets are all unmodifiable
  public Set     getAllNodes ();
  public Set     getAllEdges ();
  
  public Set     getOutNodes  (String nodeName);
  public Set     getInNodes   (String nodeName);
  public Set     getOutEdges  (String nodeName);
  public Set     getInEdges   (String nodeName);
  
  
  
  public interface Edge {
    public String getOrigin();
    public String getDestination();
    public Object getValue();
  }
  
}

